package edu.msoe.se2890.supplied;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.LinkedBlockingQueue;

import lejos.pc.comm.NXTConnector;

public class PCMindstormProxy implements Runnable {
	private static PCMindstormProxy singleton[] = { null, null };
	private NXTConnector conn;
	private DataInputStream inDat=null;
	private DataOutputStream outDat=null;
	private int proxyType;

	private BlockingQueue<Integer> transmitQueue;
	private boolean keepRunning = true;

	public static final int USBPROXY = 0;
	public static final int BLUETOOTHPROXY = 1;

	/**
	 * This method will obtain a proxy which can be used to communicate with the
	 * LegoMindstorm.
	 * 
	 * @param type
	 *            The type is either USBPROXY or BLUETOOTHPROXY.
	 * @return A proxy instance will be returned.
	 */
	public synchronized static PCMindstormProxy getSingleton(int type) {
		if ((type == USBPROXY) || (type == BLUETOOTHPROXY)) {
			if (singleton[type] == null) {
				singleton[type] = new PCMindstormProxy(type);
			}
			return singleton[type];
		} else {
			return null;
		}

	}

	/**
	 * This method will shutdown the running proxy.
	 */
	public void shutdown() {
		synchronized (this) {
			keepRunning = false;
			notifyAll();
		}
	}

	/**
	 * This method will enqueue an integer that is to be transmitted to the lego
	 * mindstorm.
	 * 
	 * @param valueToSend
	 */
	public void enqueue(int valueToSend) {
		try {
			this.transmitQueue.put(valueToSend);
			Thread.yield();
		} catch (InterruptedException e) {
		}
	}

	/**
	 * This method will receive an integer from the lego mindstorm robot.
	 * 
	 * @return The return value will be an integer from the mindstorm robot.
	 */
	public int receiveInt() {
		int retVal = Integer.MIN_VALUE;

		try {
			if (inDat == null)
			{
			synchronized(this){wait();}
			}
			retVal = inDat.readInt();
		} catch (IOException e) {
		} catch (InterruptedException e) {
		}
		return retVal;
	}

	/**
	 * This is the run method. It simply runs the proxy.
	 */
	@Override
	public void run() {
		this.startProxy();
	}

	/**
	 * This method will instantiate a new instance of the proxy that
	 * communicates with the remote device.
	 */
	private PCMindstormProxy(int type) {
		transmitQueue = new LinkedBlockingQueue<Integer>();
		proxyType = type;
	}

	/**
	 * This method will run the given proxy. It will start a connection with the
	 * remote device and start transmitting null data.
	 */
	private void startProxy() {
		conn = new NXTConnector();
		boolean connected = false;

		if (proxyType == USBPROXY) {
			connected = conn.connectTo("usb://");
		} else if (proxyType == BLUETOOTHPROXY) {
			// Connect to any NXT over Bluetooth
			connected = conn.connectTo("btspp://");
		}
		if (connected == true) {
			outDat = new DataOutputStream(conn.getOutputStream());

			Runnable transmitThread = new Runnable() {
				@Override
				public void run() {
					System.out.println("Transmission started.");
					
					while (keepRunning == true) {
						int i = 0;

						try {
							i = transmitQueue.take();
							outDat.writeInt(i);
							outDat.flush();
							System.out.println("Wrote " + i);
							Thread.yield();
						} catch (InterruptedException e) {
						} catch (IOException ioe) {
							System.err.println("IO Exception writing bytes");
						}
					}
				}
			};

			Thread tt = new Thread(transmitThread);

			tt.start();
			Thread.yield();
			inDat = new DataInputStream(conn.getInputStream());

			synchronized(this)
			{
				notify();
			}

			while (keepRunning == true) {
				synchronized (this) {
					try {
						wait();
					} catch (InterruptedException e) {
					}
				}
			}

			try {
				tt.interrupt();
				tt.join();
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

			try {
				inDat.close();
				outDat.close();
			} catch (IOException ioe) {
				System.err.println("IO Exception Closing connection");
			}

			try {
				conn.close();
			} catch (IOException ioe) {
				System.err.println("IO Exception Closing connection");
			}
		}
	}
}
